/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var BioSources = function()
{
	var biosources = {};
	
	/**
		Initialize the page.
	*/
	biosources.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', biosources.save);
			Buttons.addClickHandler('close', App.closeWindow);

			// Tab validation
			TabControl.addTabActivateListener('settings.annotations', biosources.loadAnnotationsFrame);
			TabControl.addTabValidator('settings.info', biosources.validateBioSource);
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'BIOSOURCE', 'item-id': itemId};
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnShare', Buttons.shareItem, attributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwner, attributes);
			Buttons.addClickHandler('btnNewSample', biosources.newSample, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
			
			TabControl.addTabActivateListener('main.annotations', AnnotationsList.loadOnce);
			TabControl.addTabActivateListener('main.overview', Overview.loadOnce);
			TabControl.addTabActivateListener('main.history', History.loadOnce);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'BIOSOURCE'};
			var tableAttributes = {'table-id': 'biosources'};
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnShareItems', Buttons.shareItems, tableAttributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwnerOfItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnNewItemList', biosources.newItemListOnClick);
			
			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}

	// Add event handlers to the 'new sample' icons
	biosources.initElements = function(element, autoInit)
	{
		if (autoInit == 'new-sample')
		{
			Buttons.addClickHandler(element, biosources.newSample);
		}
	}

	/*
		Create a new sample with the current biosource as the parent
	 */
	biosources.newSample = function(event)
	{
		var biosurceId = Data.int(event.currentTarget, 'item-id');
		Items.newItem('SAMPLE', '&biosource_id='+biosurceId);
	}
	
	// Create a new item list with the selected biosources as members
	biosources.newItemListOnClick = function()
	{
		var controller = Items.getController('ITEMLIST');
		Table.submitToPopup('biosources', 'CreateItemList', controller.width, controller.height);
	}
	
	biosources.validateBioSource = function()
	{
		var frm = document.forms['biosource'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		return true;
	}
	
	biosources.save = function()
	{
		var frm = document.forms['biosource'];
		if (TabControl.validateActiveTab('settings'))
		{
			Annotations.saveModifiedAnnotationsToForm(frm);
			frm.submit();
		}
	}
	
	biosources.loadAnnotationsFrame = function()
	{
		Annotations.autoLoadEditFrame(0, ItemSubtype.getSubtypeId('subtype_id'));
	}
	
	return biosources;
}();

Doc.addElementInitializer(BioSources.initElements);
Doc.onLoad(BioSources.initPage);
