/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var Projects = function()
{
	var projects = {};
	var Permission = {};
	
	/**
		Initialize the page.
	*/
	projects.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			Permission.READ = Data.int('page-data', 'read');
			Permission.USE = Data.int('page-data', 'use');
			Permission.RESTRICTED_WRITE = Data.int('page-data', 'restricted-write');
			Permission.WRITE = Data.int('page-data', 'write');
			Permission.DELETE = Data.int('page-data', 'delete');
			Permission.SET_OWNER = Data.int('page-data', 'set-owner');
			Permission.SET_PERMISSION = Data.int('page-data', 'set-permission');

			// Save + Close buttons
			Buttons.addClickHandler('btnSave', projects.save);
			Buttons.addClickHandler('close', App.closeWindow);

			// Tab validation
			TabControl.addTabValidator('settings.info', projects.validateProject);
			TabControl.addTabActivateListener('settings.annotations', Annotations.autoLoadEditFrame);

			// Members
			Buttons.addClickHandler('btnAddUsers', projects.addUsers);
			Events.addEventHandler('btnAddUsers', 'base-selected', projects.addUserCallback);
			Buttons.addClickHandler('btnAddGroups', projects.addGroups);
			Events.addEventHandler('btnAddGroups', 'base-selected', projects.addGroupCallback);
			Events.addEventHandler('members', 'change', projects.membersOnChange);
			Events.addEventHandler('members', 'base-removed', projects.memberRemoved);
		
			// Permissions
			Events.addEventHandler('read', 'click', projects.permissionsOnClick);
			Events.addEventHandler('use', 'click', projects.permissionsOnClick);
			Events.addEventHandler('restricted_write', 'click', projects.permissionsOnClick);
			Events.addEventHandler('write', 'click', projects.permissionsOnClick);
			Events.addEventHandler('delete', 'click', projects.permissionsOnClick);
			Events.addEventHandler('setOwner', 'click', projects.permissionsOnClick);
			Events.addEventHandler('setPermission', 'click', projects.permissionsOnClick);

			Events.addEventHandler('autoRead', 'click', projects.autoPermissionsOnClick);
			Events.addEventHandler('autoUse', 'click', projects.autoPermissionsOnClick);
			Events.addEventHandler('autoRestrictedWrite', 'click', projects.autoPermissionsOnClick);
			Events.addEventHandler('autoWrite', 'click', projects.autoPermissionsOnClick);
			Events.addEventHandler('autoDelete', 'click', projects.autoPermissionsOnClick);
			Events.addEventHandler('autoSetOwner', 'click', projects.autoPermissionsOnClick);
			Events.addEventHandler('autoSetPermission', 'click', projects.autoPermissionsOnClick);
			
			// Permission templates
			Buttons.addClickHandler('permissiontemplate_id.select', projects.selectPermissionTemplate);
			Events.addEventHandler('permissiontemplate_id', 'base-selected', Items.onItemSelected);
			Events.addEventHandler('permissiontemplate_id', 'change', projects.permissionTemplateOnChange);
			projects.permissionTemplateOnChange();
			var frm = document.forms['project'];
			projects.showPermissions(parseInt(frm.autoPermissions.value), 'auto_');
			
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'PROJECT', 'item-id': itemId};
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwner, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnSetActive', projects.setActive, attributes);
			
			TabControl.addTabActivateListener('main.items', projects.viewItems);
			TabControl.addTabActivateListener('main.annotations', AnnotationsList.loadOnce);
			TabControl.addTabActivateListener('main.history', History.loadOnce);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'PROJECT'};
			var tableAttributes = {'table-id': 'projects'};
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwnerOfItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);

			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
		else if (pageId == 'set-active')
		{
			Buttons.addClickHandler('close', App.closeWindow);
			window.opener.location.reload();
			setTimeout(App.closeWindow, 500);
		}
	}
	
	projects.initElements = function(element, autoInit)
	{
		if (autoInit == 'set-active')
		{
			Buttons.addClickHandler(element, projects.setActive);
		}
	}
	
	projects.setActive = function(event)
	{
		var projectId = Data.get(event.currentTarget, 'item-id');
		var url = 'set_active.jsp?ID='+App.getSessionId();
		url += '&project_id='+projectId;
		Dialogs.openPopup(url, 'SetActiveProject', 300, 200);
	}
	
	projects.viewItems = function()
	{
		var projectId = Data.get('page-data', 'item-id');
		var url = 'items/index.jsp?ID='+App.getSessionId();
		url += '&cmd=List&project_id='+projectId;
		location.replace(url);
	}

	projects.validateProject = function()
	{
		var frm = document.forms['project'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		return true;
	}

	projects.save = function()
	{
		var frm = document.forms['project'];
		if (TabControl.validateActiveTab('settings'))
		{
			Annotations.saveModifiedAnnotationsToForm(frm);
			Link.exportActions('defaultItems');
			Link.exportActions('members', '!');
			frm.submit();
		}
	}
	
	projects.getPermissionCode = function(prefix)
	{
		if (!prefix) prefix = '';
		var permissionCode = 0;
		var frm = document.forms['project'];
		if (frm[prefix+'read'].checked) permissionCode += Permission.READ;
		if (frm[prefix+'use'].checked) permissionCode += Permission.USE;
		if (frm[prefix+'restricted_write'].checked) permissionCode += Permission.RESTRICTED_WRITE;
		if (frm[prefix+'write'].checked) permissionCode += Permission.WRITE;
		if (frm[prefix+'delete'].checked) permissionCode += Permission.DELETE;
		if (frm[prefix+'set_owner'].checked) permissionCode += Permission.SET_OWNER;
		if (frm[prefix+'set_permission'].checked) permissionCode += Permission.SET_PERMISSION;
		return permissionCode;
	}
	
	projects.getPermissionString = function(permissionCode)
	{
		var s = '';
		s += permissionCode & Permission.READ ? "R" : "-";
		s += permissionCode & Permission.USE ? "U" : "-";
		s += permissionCode & Permission.RESTRICTED_WRITE ? "A" : "-";
		s += permissionCode & Permission.WRITE ? "W" : "-";
		s += permissionCode & Permission.DELETE ? "D" : "-";
		s += permissionCode & Permission.SET_OWNER ? "O" : "-";
		s += permissionCode & Permission.SET_PERMISSION ? "P" : "-";
		return s;
	}

	projects.showPermissions = function(permissionCode, prefix)
	{
		if (!prefix) prefix = '';
		var frm = document.forms['project'];
		frm[prefix+'read'].checked = permissionCode & Permission.READ;
		frm[prefix+'use'].checked = permissionCode & Permission.USE;
		frm[prefix+'restricted_write'].checked = permissionCode & Permission.RESTRICTED_WRITE;
		frm[prefix+'write'].checked = permissionCode & Permission.WRITE;
		frm[prefix+'delete'].checked = permissionCode & Permission.DELETE;
		frm[prefix+'set_owner'].checked = permissionCode & Permission.SET_OWNER;
		frm[prefix+'set_permission'].checked = permissionCode & Permission.SET_PERMISSION;
	}

	/**
		Makes sure all permission checkboxes are in a consistent state with each other.
	*/
	projects.ensureImpliedPermissions = function(checked, prefix)
	{
		if (!prefix) prefix = '';
		var frm = document.forms['project'];
		if (checked)
		{
			frm[prefix+'write'].checked = frm[prefix+'write'].checked || frm[prefix+'delete'].checked || frm[prefix+'set_owner'].checked || frm[prefix+'set_permission'].checked;
			frm[prefix+'restricted_write'].checked = frm[prefix+'restricted_write'].checked || frm[prefix+'write'].checked;
			frm[prefix+'use'].checked = frm[prefix+'use'].checked || frm[prefix+'write'].checked;
			frm[prefix+'read'].checked = frm[prefix+'read'].checked || frm[prefix+'use'].checked || frm[prefix+'restricted_write'].checked;
		}
		else
		{
			frm[prefix+'use'].checked = frm[prefix+'use'].checked && frm[prefix+'read'].checked;
			frm[prefix+'restricted_write'].checked = frm[prefix+'restricted_write'].checked && frm[prefix+'read'].checked;
			frm[prefix+'write'].checked = frm[prefix+'write'].checked && frm[prefix+'use'].checked && frm[prefix+'restricted_write'].checked;
			frm[prefix+'delete'].checked = frm[prefix+'delete'].checked && frm[prefix+'write'].checked;
			frm[prefix+'set_owner'].checked = frm[prefix+'set_owner'].checked && frm[prefix+'write'].checked;
			frm[prefix+'set_permission'].checked = frm[prefix+'set_permission'].checked && frm[prefix+'write'].checked;
		}
	}

	projects.permissionsOnClick = function(event)
	{
		var frm = document.forms['project'];
		var radio = event.currentTarget;
		projects.ensureImpliedPermissions(radio.checked);
		var permissionCode = projects.getPermissionCode();
		var permissionString = projects.getPermissionString(permissionCode);
		for (var i = 0; i < frm.members.length; i++)
		{
			var option = frm.members[i];
			if (option.selected && option.item.id)
			{
				option.item.value = permissionCode;
				var text = option.text.replace(/\[.*\]/, '['+permissionString+']');
				option.text = text; 
			}
		}
	}

	projects.autoPermissionsOnClick = function(event)
	{
		var frm = document.forms['project'];
		var radio = event.currentTarget;
		projects.ensureImpliedPermissions(radio.checked, 'auto_');
		frm.autoPermissions.value = projects.getPermissionCode('auto_');
	}
	
	/**
		Open a popup dialog for selecting users.
	*/
	projects.addUsers = function(event)
	{
		var frm = document.forms['project'];
		var current = Link.getIdsInList(frm.members, 'USER');
		var url = "&exclude="+current.join(',');
		url += '&permission=READ';
		Dialogs.selectItem('USER', event.currentTarget.id, 1, url);
	}
	
	projects.addUserCallback = function(event)
	{
		var permissionCode = projects.getPermissionCode();
		var permissionString = projects.getPermissionString(permissionCode);
		
		event.detail.name += ' ['+permissionString+']';
		event.detail.value = permissionCode;
		event.detail.originalValue = 0;
		
		Link.addItem('members', 'USER', event.detail);
	}

	/**
		Open a popup dialog for selecting groups.
	*/
	projects.addGroups = function(event)
	{
		var frm = document.forms['project'];
		var current = Link.getIdsInList(frm.members, 'GROUP');
		if (!Data.int('page-data', 'share-to-everyone'))
		{
			current[current.length] = Data.int('page-data', 'everyone-id');
		}
		var url = "&exclude="+current.join(',');
		url += '&permission=READ';
		Dialogs.selectItem('GROUP', event.currentTarget.id, 1, url);
	}
	
	projects.addGroupCallback = function(event)
	{
		var permissionCode = projects.getPermissionCode();
		var permissionString = projects.getPermissionString(permissionCode);
		
		event.detail.name += ' ['+permissionString+']';
		event.detail.value = permissionCode;
		event.detail.originalValue = 0;
		
		Link.addItem('members', 'GROUP', event.detail);
	}

	/**
		Set the permission value to 0 when an item is removed
	*/
	projects.memberRemoved = function(event)
	{
		event.detail.item.value = 0;
	}

	/**
		Update permission checkboxes for the current selected member.
	*/
	projects.membersOnChange = function()
	{
		var frm = document.forms['project'];
		var item = frm.members[frm.members.selectedIndex].item;
		projects.showPermissions(item && item.id ? item.value : 0);
	}

	projects.selectPermissionTemplate = function()
	{
		var frm = document.forms['project'];
		var url = '&resetTemporary=1';
		if (frm.permissiontemplate_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.permissiontemplate_id[1].value));
			url += '&item_id='+id;
		}
		Dialogs.selectItem('PERMISSIONTEMPLATE', 'permissiontemplate_id', 0, url);
	}
	
	projects.permissionTemplateOnChange = function()
	{
		var frm = document.forms['project'];
		var hasTemplate = frm.permissiontemplate_id.selectedIndex > 0;
		frm.auto_read.disabled = hasTemplate;
		frm.auto_use.disabled = hasTemplate;
		frm.auto_restricted_write.disabled = hasTemplate;
		frm.auto_write.disabled = hasTemplate;
		frm.auto_delete.disabled = hasTemplate;
		frm.auto_set_owner.disabled = hasTemplate;
		frm.auto_set_permission.disabled = hasTemplate;
	}

	
	return projects;
}();

Doc.onLoad(Projects.initPage);
Doc.addElementInitializer(Projects.initElements);
