/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var Plotter = function()
{
	var plotter = {};
	
	/**
		Initialize the page.
	*/
	plotter.initPage = function()
	{
		Buttons.addClickHandler('close', App.closeWindow);

		// Plot size
		Events.addEventHandler('width', 'keypress', Events.integerOnly);
		Events.addEventHandler('height', 'keypress', Events.integerOnly);

		// Aggregate
		Events.addEventHandler('yAggregateCount', 'click', plotter.aggregateOnChange);
		Events.addEventHandler('yAggregateMean', 'click', plotter.aggregateOnChange);
		
		// Plot type
		Events.addEventHandler('plotTypeScatter', 'change', plotter.plotTypeOnChange);
		Events.addEventHandler('plotTypeHistogram', 'change', plotter.plotTypeOnChange);
		plotter.plotTypeOnChange();

		// Annotations
		Events.addEventHandler('annotationPresets', 'change', plotter.annotationOnChange);
		Events.addEventHandler('annotationExp', 'change', plotter.annotationExpressionOnChange);
		
		// Buttons
		Buttons.addClickHandler('btnPreviewPlot', plotter.previewPlot);
		Buttons.addClickHandler('btnViewPlot', plotter.viewPlot);
		Buttons.addClickHandler('btnDownloadPlot', plotter.downloadPlot);
		Buttons.addClickHandler('btnSavePlot', plotter.savePlotAs);

	}

	plotter.initElements = function(element, autoInit)
	{
		if (autoInit == 'formula-presets')
		{
			Events.addEventHandler(element, 'change', plotter.presetOnChange);
		}
		else if (autoInit == 'expression-builder')
		{
			Buttons.addClickHandler(element, plotter.openExpressionBuilder);
		}
	}
	
	plotter.getPlotType = function()
	{
		var frm = document.forms['plot'];
		return Forms.getCheckedRadio(frm.plotType).value;
	}

	
	plotter.validateParameters = function()
	{
		var plotType = plotter.getPlotType();
		if (plotType == 'scatter')
		{
			return plotter.validateScatterPlot();
		}
		else if (plotType == 'histogram')
		{
			return plotter.validateHistogramPlot();
		}
		return false;
	}
	
	plotter.validateScatterPlot = function()
	{
		var frm = document.forms['plot'];
		if (Strings.trim(frm.yFormulaScatter.value) == '')
		{
			Forms.showNotification(frm.yFormulaScatter, 'You must enter an expression for the Y axis');
			return false;
		}
		else if (Strings.trim(frm.xFormulaScatter.value) == '')
		{
			Forms.showNotification(frm.xFormulaScatter, 'You must enter an expression for the X axis');
			return false;
		}
		return true;
	}
	
	plotter.validateHistogramPlot = function()
	{
		var frm = document.forms['plot'];
		var yAggregate = Forms.getCheckedRadio(frm.yAggregate).value;
		if (Strings.trim(frm.xFormulaHistogram.value) == '')
		{
			Forms.showNotification(frm.xFormulaHistogram, 'You must enter an expression for the X axis');
			return false;
		}
		else if (yAggregate != 'count' && Strings.trim(frm.yFormulaHistogram.value) == '')
		{
			Forms.showNotification(frm.yFormulaHistogram, 'You must enter an expression for the Y axis');
			return false;
		}
		return true;
	}

	plotter.plotTypeOnChange = function()
	{
		var isScatter = plotter.getPlotType() == 'scatter';
		Doc.showHide('scatterSection', isScatter);
		Doc.showHide('histogramSection', !isScatter);
	}

	plotter.presetOnChange = function(event)
	{
		var list = event.currentTarget;
		var selected = list[list.selectedIndex];
		
		var formulaField = Data.get(list, 'formula-id');
		Doc.element(formulaField).value = selected.value;

		var labelField = Data.get(list, 'label-id');
		if (labelField && selected.value != '') Doc.element(labelField).value = selected.text;
		list.selectedIndex = 0;
	}

	
	plotter.openExpressionBuilder = function(event)
	{
		var formulaField = Doc.element(Data.get(event.currentTarget, 'formula-id'));
		
		if (!formulaField.disabled)
		{
			var title = Data.get(event.currentTarget, 'title');
			var formulaType = Data.get(event.currentTarget, 'formula-type');
			var rawDataType = Data.get(event.currentTarget, 'raw-data-type');
			var channels = Data.int(event.currentTarget, 'channels');
			var bioAssaySetId = Data.int(event.currentTarget, 'bioassayset');
			
			Dialogs.openExpressionBuilder(formulaField, title, formulaType, rawDataType, channels, bioAssaySetId);
		}
	}
	
	plotter.aggregateOnChange = function()
	{
		var frm = document.forms['plot'];
		var selected = Forms.getCheckedRadio(frm.yAggregate);
		var isCount = selected.value == 'count';
		frm.yFormulaHistogram.disabled = isCount;
		frm.yPresetsHistogram.disabled = isCount;
		frm.yLogHistogram.disabled = isCount;
		for (var i = 0; i < frm.hiloAggregate.length; i++)
		{
			frm.hiloAggregate[i].disabled = isCount;
		}
		if (isCount)
		{
			Doc.removeClass(frm.yFormulaHistogram, 'required');
			frm.yLabelHistogram.value = 'Count';
		}
		else
		{
			Doc.addClass(frm.yFormulaHistogram, 'required');
			var yPreset = frm.yPresetsHistogram[frm.yPresetsHistogram.selectedIndex];
			if (yPreset.value != '') frm.yLabelHistogram.value = yPresetHistogram.text;
		}
	}

	plotter.annotationOnChange = function(event)
	{
		var list = event.currentTarget;
		var frm = document.forms['plot'];
		var selected = list[list.selectedIndex];
		var value = selected.value && selected.value != '$' ? selected.text : '';
		value = value.replace('\[A\] ', '');
		frm.subTitle.value = value;
		
		if (selected.value.substring(0, 1) == '$')
		{
			frm.annotationExp.disabled = true;
		}
		else
		{
			if (selected.value)
			{
				frm.annotationExp.value = selected.value;
			}
			frm.annotationExp.disabled = false;
			frm.annotationExp.focus();
		}
	}
	
	plotter.annotationExpressionOnChange = function()
	{
		var frm = document.forms['plot'];
		frm.annotationPresets.selectedIndex = frm.annotationPresets.length-1;
	}
	

	
	plotter.generatePlotUrl = function(fullSize)
	{
		if (plotter.validateParameters())
		{
			var frm = document.forms['plot'];
			var url = 'plot?ID='+App.getSessionId();
			url += '&bioassayset_id='+Data.get('page-data', 'bioassayset');
			url += '&bioassay_id='+Data.get('page-data', 'bioassay');
			url += '&title='+encodeURIComponent(frm.title.value);
			url += '&subTitle='+encodeURIComponent(frm.subTitle.value);
			if (fullSize)
			{
				url += '&width='+frm.width.value;
				url += '&height='+frm.height.value;
			}
			
			if (frm.filter.value != '')
			{
				url += '&filter='+encodeURIComponent(frm.filter.value);
			}
			
			var annotation = '';
			if (!frm.annotationExp.disabled)
			{
				if (frm.annotationExp.value)
				{
					annotation = '=' + frm.annotationExp.value;
				}
			}
			else
			{
				annotation = frm.annotationPresets.value.substring(1);
			}
			if (annotation)
			{
				url += '&annotation='+encodeURIComponent(annotation);
			}

			var plotType = plotter.getPlotType();
			if (plotType == 'scatter')
			{
				url += '&type=scatter';
				url += '&x='+encodeURIComponent(frm.xFormulaScatter.value);
				url += '&xLabel='+encodeURIComponent(frm.xLabelScatter.value);
				url += '&xLog='+(frm.xLogScatter.checked ? 1 : 0);
				url += '&y='+encodeURIComponent(frm.yFormulaScatter.value);
				url += '&yLabel='+encodeURIComponent(frm.yLabelScatter.value);
				url += '&yLog='+(frm.yLogScatter.checked ? 1 : 0);
			}
			else
			{
				var yAggregate = Forms.getCheckedRadio(frm.yAggregate).value;
				url += '&type=histogram';
				url += '&x='+encodeURIComponent(frm.xFormulaHistogram.value);
				url += '&xLabel='+encodeURIComponent(frm.xLabelHistogram.value);
				url += '&xLog='+(frm.xLogHistogram.checked ? 1 : 0);
				url += '&binSize='+encodeURIComponent(frm.binSize.value);
				url += '&yAggregate='+yAggregate;
				url += '&yLabel='+encodeURIComponent(frm.yLabelHistogram.value);
				if (yAggregate != 'count')
				{
					url += '&y='+encodeURIComponent(frm.yFormulaHistogram.value);
					url += '&yLog='+(frm.yLogHistogram.checked ? 1 : 0);
					url += '&hiloAggregate='+Forms.getCheckedRadio(frm.hiloAggregate).value
				}
			}
			return url;
		}
	}

	plotter.previewPlot = function()
	{
		var url = plotter.generatePlotUrl(false);
		if (url)
		{
			url += '&width=540&height=360';

			var overlayImg = Doc.element('overlay');
			overlayImg.src = App.getRoot()+'images/plot_generating.gif';
			
			var tmpPlot = new Image();
			Events.addEventHandler(tmpPlot, 'load', plotter.previewLoaded);
			tmpPlot.src = url;
		}
	}

	plotter.previewLoaded = function(event)
	{
		var tmpPlot = event.currentTarget;
		
		var previewImg = Doc.element('preview');
		previewImg.src = tmpPlot.src;

		var overlayImg = Doc.element('overlay');
		overlayImg.src = App.getRoot() + 'images/blankbutton.gif';
	}

	plotter.viewPlot = function()
	{
		var plotUrl = plotter.generatePlotUrl(true);
		if (plotUrl)
		{
			var frm = document.forms['plot'];
			var width = parseInt(frm.width.value);
			var height = parseInt(frm.height.value);
			if (!width || width < 600) width = 600;
			if (!height || height < 400) height = 400;
			var url = 'view.jsp?ID='+App.getSessionId();
			url += '&title='+encodeURIComponent(frm.title.value);
			url += '&plot='+encodeURIComponent(plotUrl);
			Dialogs.openPopup(url, 'ViewPlot', width+150, height+100);
		}
	}

	
	plotter.downloadPlot = function()
	{
		var plotUrl = plotter.generatePlotUrl(true);
		if (plotUrl)
		{
			var url = 'download.jsp?ID='+App.getSessionId();
			url += '&plot='+encodeURIComponent(plotUrl);
			Dialogs.openPopup(url, 'DownloadPlot', 300, 200);
		}
	}

	plotter.savePlotAs = function()
	{
		var plotUrl = plotter.generatePlotUrl(true);
		if (plotUrl)
		{
			var url = 'save_as.jsp?ID='+App.getSessionId();
			url += '&plot='+encodeURIComponent(plotUrl);
			Dialogs.openPopup(url, 'SavePlotAs', 450, 300);
		}
	}


	return plotter;
}();

Doc.onLoad(Plotter.initPage);
Doc.addElementInitializer(Plotter.initElements);

